@@ -4,14 +4,14 @@ use strict;
 use Mouse;
 use Authen::Radius;
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_BADCREDENTIALS
   PE_OK
+  PE_BADCREDENTIALS
   PE_RADIUSCONNECTFAILED
 );
 
 extends qw(Lemonldap::NG::Portal::Auth::_WebForm);
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.0.14';
 
 # PROPERTIES
 
@@ -39,24 +39,28 @@ sub initRadius {
 sub init {
     my $self = shift;
     unless ( $self->initRadius ) {
-        $self->error('Radius connect failed');
+        $self->error('Radius initialisation failed');
     }
-    return 1;
+
+    return $self->Lemonldap::NG::Portal::Auth::_WebForm::init();
 }
 
 # RUNNING METHODS
 
 sub authenticate {
     my ( $self, $req ) = @_;
-    $self->initRadius unless ( $self->radius );
+    $self->initRadius unless $self->radius;
     unless ( $self->radius ) {
         $self->setSecurity($req);
         return PE_RADIUSCONNECTFAILED;
     }
 
+    $self->logger->debug(
+"Send authentication request ($req->{user}) to Radius server ($self->{conf}->{radiusServer})"
+    );
     my $res = $self->radius->check_pwd( $req->user, $req->data->{password} );
     unless ( $res == 1 ) {
-        $self->userLogger->warn("Unable to authenticate $req->{user} !");
+        $self->userLogger->warn("Unable to authenticate $req->{user}!");
         $self->setSecurity($req);
         return PE_BADCREDENTIALS;
     }
