@@ -4,13 +4,18 @@ use strict;
 use Mouse;
 
 # Add constants used by this module
-use Lemonldap::NG::Portal::Main::Constants
-  qw(PE_OK PE_FORBIDDENIP PE_USERNOTFOUND);
-use Lemonldap::NG::Portal::Lib::Slave;
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_FORBIDDENIP
+  PE_USERNOTFOUND
+);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.12';
 
-extends 'Lemonldap::NG::Portal::Auth::Base';
+extends qw(
+  Lemonldap::NG::Portal::Main::Auth
+  Lemonldap::NG::Portal::Lib::Slave
+);
 
 # INITIALIZATION
 
@@ -23,6 +28,11 @@ sub extractFormInfo {
     return PE_FORBIDDENIP
       unless ( $self->checkIP($req) and $self->checkHeader($req) );
 
+    unless ( $self->conf->{slaveUserHeader} ) {
+        $self->logger->debug('slaveUserHeader is undefined');
+        return PE_USERNOTFOUND;
+    }
+
     my $user_header = $self->conf->{slaveUserHeader};
     $user_header = 'HTTP_' . uc($user_header);
     $user_header =~ s/\-/_/g;
@@ -32,23 +42,28 @@ sub extractFormInfo {
             "No header " . $self->conf->{slaveUserHeader} . " found" );
         return PE_USERNOTFOUND;
     }
-    PE_OK;
+    return PE_OK;
 }
 
 sub authenticate {
     my ( $self, $req ) = @_;
-    PE_OK;
+    return PE_OK;
 }
 
 sub setAuthSessionInfo {
     my ( $self, $req ) = @_;
     $req->{sessionInfo}->{authenticationLevel} = $self->conf->{slaveAuthnLevel};
-    PE_OK;
+    return PE_OK;
+}
+
+sub getDisplayType {
+    my ($self) = @_;
+    return ( $self->{conf}->{slaveDisplayLogo} ? "logo" : "_none_" );
 }
 
 sub authLogout {
     my ( $self, $req ) = @_;
-    PE_OK;
+    return PE_OK;
 }
 
 1;
