@@ -9,7 +9,7 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '2.0.14';
+our $VERSION = '2.0.15';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -121,7 +121,20 @@ sub handler {
 
 sub authenticated {
     my ( $self, $req ) = @_;
-    return $self->sendJSONresponse( $req, { status => 1 } );
+
+    return $self->do(
+        $req,
+        [
+            'importHandlerData',
+            'controlUrl',
+            @{ $self->forAuthUser },
+            sub {
+                $req->response(
+                    $self->sendJSONresponse( $req, { status => 1 } ) );
+                return PE_SENDRESPONSE;
+            }
+        ]
+    );
 }
 
 sub pleaseAuth {
@@ -159,6 +172,7 @@ sub postLogin {
 
 sub authenticatedRequest {
     my ( $self, $req ) = @_;
+    $req->data->{alreadyAuthenticated} = 1;
     return $self->do(
         $req,
         [
@@ -189,8 +203,27 @@ sub refresh {
     $req->user( $data{_user} || $data{ $self->conf->{whatToTrace} } );
     $req->id( $data{_session_id} );
     foreach ( keys %data ) {
-        delete $data{$_}
-          unless ( /^_/ or /^(?:startTime|authenticationLevel)$/ );
+
+        # Variables that start with _ are kept accross refresh
+        if (/^_/) {
+
+            # But not OIDC tokens, which can be refreshed
+            delete $data{$_}
+              if (
+/^(_oidc_access_token|_oidc_refresh_token|_oidc_access_token_eol)$/
+              );
+        }
+
+        # Other variables should be refreshed
+        else {
+            # But not these two
+            if (/^(?:startTime|authenticationLevel)$/) {
+                next;
+            }
+            else {
+                delete $data{$_};
+            }
+        }
     }
     $data{_updateTime} = strftime( "%Y%m%d%H%M%S", localtime() );
     $self->logger->debug(
@@ -265,19 +298,15 @@ sub do {
     # Update status
     if ( my $p = $self->HANDLER->tsv->{statusPipe} ) {
         $p->print( ( $req->user ? $req->user : $req->address ) . ' => '
-              . $req->uri
+              . $req->{env}->{REQUEST_URI}
               . " $err\n" );
     }
 
     # Update history
-    if ( $err == PE_SENDRESPONSE ) {
-        return $req->response;
-    }
+    return $req->response if $err == PE_SENDRESPONSE;
 
     # Remove userData if authentication fails
-    if ( $err == PE_BADCREDENTIALS or $err == PE_BADOTP ) {
-        $req->userData( {} );
-    }
+    $req->userData( {} ) if ( $err == PE_BADCREDENTIALS or $err == PE_BADOTP );
 
     if ( !$self->conf->{noAjaxHook} and $req->wantJSON ) {
         $self->logger->debug('Processing to JSON response');
@@ -306,24 +335,26 @@ sub do {
         elsif ( $err > 0 and $err != PE_PASSWORD_OK and $err != PE_LOGOUT_OK ) {
             return $self->sendJSONresponse(
                 $req,
-                { result => 0, error => $err },
+                {
+                    result => 0,
+                    error  => $err,
+                    (
+                        $err == PE_NOTIFICATION && $req->id
+                        ? ( ciphered_id => $req->id )
+                        : ()
+                    )
+                },
                 code => 400
             );
         }
         else {
-            return $self->sendJSONresponse(
-                $req,
-                {
-                    result => 1,
-                    error  => $err,
-                    id     => $req->id,
-                    (
-                        $req->sessionInfo->{_httpSession}
-                        ? ( id_http => $req->sessionInfo->{_httpSession} )
-                        : ()
-                    )
-                }
-            );
+            my $res = { result => 1, error => $err };
+            unless ( $req->data->{alreadyAuthenticated} ) {
+                $res->{id}      = $req->id;
+                $res->{id_http} = $req->sessionInfo->{_httpSession}
+                  if $req->sessionInfo->{_httpSession};
+            }
+            return $self->sendJSONresponse( $req, $res );
         }
     }
     else {
@@ -458,7 +489,8 @@ sub getApacheSession {
         $self->logger->debug("Session $args{kind} $id not found");
         return;
     }
-    $self->logger->debug("Get session $id from Portal::Main::Run") if ($id);
+    $self->logger->debug("Get session $id from Portal::Main::Run")
+      if ($id);
     $self->logger->debug(
         "Check session validity  -> " . $self->conf->{timeoutActivity} . "s" )
       if ( $self->conf->{timeoutActivity} );
@@ -490,7 +522,9 @@ sub getApacheSession {
 sub getPersistentSession {
     my ( $self, $uid, $info ) = @_;
 
-    return unless ( defined $uid and !$self->conf->{disablePersistentStorage} );
+    return
+      unless ( defined $uid
+        and !$self->conf->{disablePersistentStorage} );
 
     # Compute persistent identifier
     my $pid = getPSessionID($uid);
@@ -838,11 +872,58 @@ sub _dump {
     return;
 }
 
+# Warning: this function returns a JSON string
+sub getTrOver {
+    my ( $self, $req, $templateDir ) = @_;
+
+    my $templateDir //= $self->conf->{templateDir} . '/' . $self->getSkin($req);
+
+    unless ( $self->trOverCache->{$templateDir} ) {
+
+        # Override messages
+        my $trOverMessages = JSON::from_json( $self->trOver );
+
+        opendir( DIR, $templateDir );
+        my @langfiles = grep( /\.json$/, readdir(DIR) );
+        close(DIR);
+
+        foreach my $file (@langfiles) {
+            my ($lang) = ( $file =~ /^(\w+)\.json/ );
+            $self->logger->debug("Use $file to override messages");
+            if ( open my $json, "<", $templateDir . "/" . $file ) {
+                my $trdata;
+                eval {
+                    local $/ = undef;
+                    $trdata = JSON::from_json(<$json>);
+                };
+                if ($@) {
+                    $self->logger->warn("Ignoring $file because of error: $@");
+                }
+                if ( ref($trdata) eq "HASH" ) {
+                    for my $msg ( keys %$trdata ) {
+
+                        # lemonldap-ng.ini has priority
+                        $trOverMessages->{$lang}->{$msg} //= $trdata->{$msg};
+                    }
+                }
+            }
+            else {
+                $self->logger->error("Unable to read $file");
+            }
+        }
+
+        $self->trOverCache->{$templateDir} = JSON::to_json($trOverMessages);
+    }
+
+    return $self->trOverCache->{$templateDir};
+}
+
 sub sendHtml {
     my ( $self, $req, $template, %args ) = @_;
 
     my $templateDir = $self->conf->{templateDir} . '/' . $self->getSkin($req);
-    $self->templateDir( [ $templateDir, @{ $self->templateDir } ] );
+    my $defaultTemplateDir = $self->conf->{templateDir} . '/bootstrap';
+    $self->templateDir( [ $templateDir, $defaultTemplateDir ] );
 
     # Check template
     $args{templateDir} = $templateDir;
@@ -854,29 +935,7 @@ sub sendHtml {
         $self->logger->debug("-> Trying to load $tmpl");
     }
 
-    # Override messages
-    my $trOverMessages = JSON::from_json( $self->trOver );
-
-    unless ( $self->trOverCache->{$templateDir} ) {
-        opendir( DIR, $templateDir );
-        my @langfiles = grep( /\.json$/, readdir(DIR) );
-        close(DIR);
-
-        foreach my $file (@langfiles) {
-            my ($lang) = ( $file =~ /^(\w+)\.json/ );
-            $self->logger->debug("Use $file to override messages");
-            if ( open my $json, "<", $templateDir . "/" . $file ) {
-                local $/ = undef;
-                $trOverMessages->{$lang} = JSON::from_json(<$json>);
-            }
-            else {
-                $self->logger->error("Unable to read $file");
-            }
-        }
-
-        $self->trOverCache->{$templateDir} = JSON::to_json($trOverMessages);
-    }
-    $args{params}->{TROVER} = $self->trOverCache->{$templateDir};
+    $args{params}->{TROVER} = $self->getTrOver( $req, $templateDir );
 
     my $res = $self->SUPER::sendHtml( $req, $template, %args );
     push @{ $res->[1] },
@@ -888,21 +947,32 @@ sub sendHtml {
 
     $self->setCorsHeaderFromConfig($res);
 
+    if (    $self->conf->{strictTransportSecurityMax_Age}
+        and $self->conf->{portal} =~ /^https:/ )
+    {
+        push @{ $res->[1] },
+          'Strict-Transport-Security' =>
+          "max-age=$self->{conf}->{strictTransportSecurityMax_Age}";
+        $self->logger->debug(
+"Set Strict-Transport-Security with: $self->{conf}->{strictTransportSecurityMax_Age}"
+        );
+    }
+
     # Set authorized URL for POST
     my $csp = $self->csp . "form-action " . $self->conf->{cspFormAction};
     if ( my $url = $req->urldc ) {
-        $self->logger->debug("Required urldc : $url");
+        $self->logger->debug("Required urldc: $url");
         $url =~ URIRE;
         $url = $2 . '://' . $3 . ( $4 ? ":$4" : '' );
-        $self->logger->debug("Set CSP form-action with urldc : $url");
+        $self->logger->debug("Set CSP form-action with urldc: $url");
         $csp .= " $url";
     }
     my $url = $args{params}->{URL};
     if ( defined $url ) {
-        $self->logger->debug("Required Params URL : $url");
+        $self->logger->debug("Required Params URL: $url");
         if ( $url =~ URIRE ) {
             $url = $2 . '://' . $3 . ( $4 ? ":$4" : '' );
-            $self->logger->debug("Set CSP form-action with Params URL : $url");
+            $self->logger->debug("Set CSP form-action with Params URL: $url");
             $csp .= " $url";
         }
     }
@@ -923,7 +993,6 @@ sub sendHtml {
     {
         $self->logger->debug(
             "Add SAML Discovery Protocol URL in CSP form-action");
-
         $csp .= " " . $self->conf->{samlDiscoveryProtocolURL};
     }
     $csp .= ';';
@@ -961,7 +1030,7 @@ sub sendHtml {
 
     # Set CSP header
     push @{ $res->[1] }, 'Content-Security-Policy' => $csp;
-    $self->logger->debug("Apply following CSP : $csp");
+    $self->logger->debug("Apply following CSP: $csp");
     return $res;
 }
 
@@ -1046,13 +1115,21 @@ sub tplParams {
     }
 
     return (
-        SKIN       => $self->getSkin($req),
         PORTAL_URL => $self->conf->{portal},
+        MAIN_LOGO  => $self->conf->{portalMainLogo},
+        LANGS      => $self->conf->{showLanguages},
+        SCROLL_TOP => $self->conf->{scrollTop},
+        SKIN       => $self->getSkin($req),
         SKIN_PATH  => $portalPath . "skins",
         SAMESITE   => getSameSite( $self->conf ),
         SKIN_BG    => $self->conf->{portalSkinBackground},
+        FAVICON    => $self->conf->{portalFavicon} || 'common/favicon.ico',
         CUSTOM_CSS => $self->conf->{portalCustomCss},
-        ( $self->customParameters ? ( %{ $self->customParameters } ) : () ),
+        (
+            $self->customParameters
+            ? ( %{ $self->customParameters } )
+            : ()
+        ),
         %templateParams
     );
 }
@@ -1163,7 +1240,9 @@ sub sendJSONresponse {
     # If this is a cross-domain request from the portal itself
     # (Ajax SSL to a different VHost)
     # we allow CORS
-    if ( $req->origin and index( $self->conf->{portal}, $req->origin ) == 0 ) {
+    if ( $req->origin
+        and index( $self->conf->{portal}, $req->origin ) == 0 )
+    {
         $self->logger->debug('AJAX request from portal, allowing CORS');
         push @{ $res->[1] },
           "Access-Control-Allow-Origin"      => $req->origin,
@@ -1192,7 +1271,7 @@ sub setCorsHeaderFromConfig {
     if ( $self->conf->{corsEnabled} ) {
         my @cors = split /;/, $self->cors;
         push @{ $response->[1] }, @cors;
-        $self->logger->debug('Apply following CORS policy :');
+        $self->logger->debug('Apply following CORS policy:');
         $self->logger->debug(" $_") for @cors;
     }
 }
