@@ -14,7 +14,7 @@ extends qw(
   Lemonldap::NG::Portal::Password::Base
 );
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.0.15';
 
 sub init {
     my ($self) = @_;
@@ -63,10 +63,7 @@ sub modifyPassword {
     my $code =
       $self->ldap->userModifyPassword( $dn, $pwd, $req->data->{oldpassword},
         0, $requireOldPassword );
-
-    unless ( $code == PE_PASSWORD_OK ) {
-        return $code;
-    }
+    return $code unless ( $code == PE_PASSWORD_OK );
 
     # If password policy and force reset, set reset flag
     if (    $self->conf->{ldapPpolicyControl}
