@@ -3,30 +3,34 @@ package Lemonldap::NG::Portal::UserDB::WebID;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
   PE_ERROR
   PE_MISSINGREQATTR
-  PE_OK
 );
 
 extends 'Lemonldap::NG::Common::Module';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.12';
 
 # INITIALIZATION
 
 sub init {
-    1;
+    return 1;
 }
 
 # RUNNING METHODS
 
 sub getUser {
-    PE_OK;
+    return PE_OK;
+}
+
+sub findUser {
+    return PE_OK;
 }
 
 sub setSessionInfo {
     my ( $self, $req ) = @_;
-    unless ( $req->datas->{_webid} ) {
+    unless ( $req->data->{_webid} ) {
         $self->logger->error('No webid object found');
         return PE_ERROR;
     }
@@ -39,7 +43,7 @@ sub setSessionInfo {
         my $attr = $k;
         my $req;
         $attr =~ s/^!// and $req = 1;
-        eval { $req->{sessionInfo}->{$attr} = $req->datas->{_webid}->get($v) };
+        eval { $req->{sessionInfo}->{$attr} = $req->data->{_webid}->get($v) };
         $self->logger->error("Unable to get $v from FOAF document: $@")
           if ($@);
         if ( $req and not $req->{sessionInfo}->{$attr} ) {
@@ -48,11 +52,12 @@ sub setSessionInfo {
             return PE_MISSINGREQATTR;
         }
     }
-    PE_OK;
+
+    return PE_OK;
 }
 
 sub setGroups {
-    PE_OK;
+    return PE_OK;
 }
 
 1;
